/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.backend;

import java.util.ArrayList;
import java.util.List;

import at.dms.classfile.Constants;
import at.dms.classfile.LocalVarInstruction;
import at.dms.classfile.MethodInfo;

/**
 * This class represents a method environment.
 */
class MethodEnv
{
    /**
     * Locals.
     */
    private List locals = new ArrayList();

    /**
     * MethodInfo from classfile.
     */
    private MethodInfo info;

    MethodEnv(MethodInfo info)
    {
        this.info = info;
    }

    /**
     * getLocalVar
     */
    public QTemporary getLocalVar(int pos)
    {
        return (QTemporary)locals.get(pos);
    }

    /**
     * getLocalVar
     */
    public QTemporary getLocalVar(InstructionHandle instruction)
    {
        int slot =
            ((LocalVarInstruction)instruction.getInstruction()).getIndex();

        if (slot >= locals.size()) {
            while (slot >= locals.size()) {
                locals.add(null);
            }
        }

        QTemporary temp = (QTemporary)locals.get(slot);

        if (temp == null) {
            instruction.getInstruction().dump();
            temp = new QTemporary(instruction.getInstruction().getReturnType(),
                                  slot);
            locals.set(slot, temp);
        }

        return temp;
    }

    public void setParameters(int[] sizes)
    {
        getParameters(info, info.getSignature(), sizes);
    }

    /**
     * Computes the number of parameters. (MOVE FROM HERE !!!)
     */
    private static void getParameters(MethodInfo info,
                                      String signature,
                                      int[] sizes)
    {
        int paramCnt = 0;

        if (paramCnt == sizes.length) {
            return;
        }

        if ((info.getModifiers() & Constants.ACC_STATIC) == 0) {
            /* An instance method always passes "this" as first, hidden
             * parameter.
             */
            sizes[paramCnt++] = 1;
        }

        if (signature.charAt(0) != '(') {
            throw new InternalError("invalid signature " + signature);
        }

        int pos = 1;

    _method_parameters_:
        for (;;) {
            if (paramCnt == sizes.length) {
                return;
            }
            switch (signature.charAt(pos++)) {
            case ')':
                break _method_parameters_;

            case '[':
                while (signature.charAt(pos) == '[') {
                    pos += 1;
                }
                if (signature.charAt(pos) == 'L') {
                    while (signature.charAt(pos) != ';') {
                        pos += 1;
                    }
                }
                pos += 1;

                sizes[paramCnt++] = 1;
                break;

            case 'L':
                while (signature.charAt(pos) != ';') {
                    pos += 1;
                }
                pos += 1;

                sizes[paramCnt++] = 1;
                break;

            case 'Z':
            case 'B':
            case 'C':
            case 'S':
            case 'F':
            case 'I':
                sizes[paramCnt++] = 1;
                break;

            case 'D':
            case 'J':
                sizes[paramCnt++] = 2;
                break;

            default:
                throw new InternalError("invalid signature " + signature);
            }
        }
    }
}
