/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.backend;

import java.util.ArrayList;
import java.util.List;

import at.dms.classfile.OpcodeNames;

/**
 * ???
 */
class QOperator
    implements QOrigin
{
    /**
     */
    private InstructionHandle operator;

    /**
     */
    private QOrigin[] operands;

    public QOperator(InstructionHandle operator, QOrigin[] operands)
    {
        this.operator = operator;
        this.operands = operands;
    }

    /**
     * Human readable form.
     */
    public String toString()
    {
        StringBuffer s = new StringBuffer();
        s.append(OpcodeNames.getName(operator.getOpcode())).append(" {");
    
        for (int i = operands.length - 1; i >= 0; i--) {
            s.append(operands[i]);
            if (i != 0) {
                s.append(", ");
            }
        }
        s.append('}');
        return s.toString();
    }

    /**
     * The type of this instruction.
     */
    public int getType()
    {
        return operator.getInstruction().getReturnType();
    }

    /**
     * Returns the primitive instruction.
     */
    public InstructionHandle getInstruction()
    {
        return operator;
    }  

    /**
     * Duplicate this node: not implemented.
     */
    public QOrigin duplicate()
    {
        throw new RuntimeException("NYI");
    }

    /**
     * Returns the used temporaries.
     */
    public QTemporary[] getUses()
    {
        List list = new ArrayList();

        for ( int i = 0; i < operands.length; i++) {
            if ((operands[i] instanceof QTemporary)
                && (!list.contains(operands[i]))) {

                list.add(operands[i]);
            }
        }

        return (QTemporary[])list.toArray(new QTemporary[list.size()]);
    }

    /**
     * Returns the parameters of this instruction.
     */
    public QOrigin[] getOrigins()
    {
        return operands;
    }

    /**
     * Sets the parameters of this instruction.
     */
    public void setOrigin(QOrigin origin, int i)
    {
        operands[i] = origin;
    }

    /**
     * Generates instructions for this quadruple.
     *
     * @param seq
     * The code sequence of instruction.
     */
    public void generate(CodeSequence seq)
    {
        for (int i = operands.length - 1; i >= 0; i--) {
            operands[i].generate(seq);
        }
        seq.plantInstruction(operator);
    }
}
