/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package at.dms.kjc;

import java.io.File;
import java.util.List;
import at.dms.classfile.FieldInfo;

/**
 * This class represents an undefined class (something that comes from a bad
 * classfile.) This class is not usable to anything, so it will sooner or
 * later produce a comprehensive error.
 */
public class CBadClass extends CClass
{

    /**
     * Constructs a class export from file
     */
    public CBadClass(String qualifiedName)
    {
        super(null, null, "undefined", 0,
              getIdent(qualifiedName),
              qualifiedName,
              null, false);
    }

    /**
     * descendsFrom
     * @param from an other CClass
     * @return true if this class inherit from "from" or equals "from"
     */
    public boolean descendsFrom(CClass from)
    {
        return false;
    }

    /**
     * @param ident  the name of the field
     * @return the field
     */
    public final CField getField(String ident)
    {
        return null;
    }

    /**
     * This can be used to see if a given class name is visible inside of
     * this file.  This includes globally-qualified class names that
     * include their package and simple names that are visible thanks to some
     * previous import statement or visible because they are in this file.
     * If one is found, that entry is returned, otherwise null is returned.
     * @param name  a TypeName (6.5.2)
     */
    public CClass lookupClass(String name)
    {
        return null;
    }

    /**
     * lookupMethod
     * search for a matching method with the provided type parameters
     * look in parent hierarchy as needed
     * @param name  method name
     * @param params  method parameters
     * @exception UnpositionedError this error will be positioned soon
     */
    public CMethod lookupMethod(String name, CType[] params)
    {
        return null;
    }

    /**
     * lookupSuperMethod
     * search for a matching method with the provided type parameters
     * look in parent hierarchy as needed
     * @param name  method name
     * @param params  method parameters
     * @exception UnpositionedError this error will be positioned soon
     */
    public CMethod lookupSuperMethod(String name, CType[] params)
    {
        return null;
    }

    /**
     * lookupField
     * search for a field
     * look in parent hierarchy as needed
     * @param name  method name
     * @param params  method parameters
     * @exception UnpositionedError this error will be positioned soon
     */
    public CField lookupField(String name)
    {
        return null;
    }

    /**
     * @return true if this member is accessible
     */
    public boolean isAccessible(CClass from)
    {
        return false;
    }

    /**
     * Returns a list of interface methods
     */
    public CMethod[] getInterfaceMethods()
    {
        return new CMethod[0];
    }

    /**
     * Returns a list of abstract methods
     */
    public CMethod[] getAbstractMethods()
    {
        return new CMethod[0];
    }

    /**
     * collectInterfaceMethods
     * search for a matching method with the provided type parameters
     * look in parent hierarchy as needed
     * @param name  method name
     * @param params  method parameters
     */
    public void collectInterfaceMethods(List l)
    {
    }

    /**
     * collectAbstractMethods
     * search for a matching method with the provided type parameters
     * look in parent hierarchy as needed
     * @param name  method name
     * @param params  method parameters
     */
    public void collectAbstractMethods(List l)
    {
    }

    /**
     * Generate the code in a class file
     * @param classes  a vector to add inner classes
     */
    public void genClassFile(File destination)
    {
        throw new RuntimeException();
    }
}
