/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.kjc;

import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;

/**
 * This class represents a constructor method context during check.
 *
 * @see CCompilationUnitContext
 * @see CClassContext
 * @see CMethodContext
 * @see CContext
 */
public class CConstructorContext
    extends CMethodContext
{
    /**
     * Whether this constructor calls its super-class constructor.
     * If it doesn't explicitly do that, it must be done implicitly by the
     * compiler.
     */
    private boolean isSuperConstructorCalled;

    /**
     * Makes a new CConstructorContext object.
     *
     * @param parent
     * The parent context.
     *
     * @param self
     * The corresponding method interface.
     */
    public CConstructorContext(CClassContext parent, CMethod self)
    {
        super(parent, self);

        isSuperConstructorCalled = true;
    }

    /**
     * Verify that all checked exceptions are defined in the throw list.
     *
     * @exception PositionedError
     * Error.
     */
    public void close(TokenReference ref) 
        throws PositionedError
    {
        if (getClassContext().getCClass().isAnonymous()) {
            getCMethod().setThrowables(getThrowables());
        }

        super.close(ref);
    }

    /**
     * Marks the super-constructor as being called by this constructor, or not.
     */
    public void setSuperConstructorCalled(boolean b)
    {
        this.isSuperConstructorCalled = b;
    }

    /**
     * Returns whether the super constructor is called by this constructor,
     * or not.
     */
    public boolean isSuperConstructorCalled()
    {
        return isSuperConstructorCalled;
    }
}
