/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package at.dms.kjc;

/**
 * This class represents a local context during checkBody
 * It follows the control flow and maintain informations about
 * variable (initialised, used, allocated), exceptions (thrown, catched)
 * It also verify that context is still reachable
 *
 * There is a set of utilities method to access fields, methods and class
 * with the name by clamping the parsing tree
 * @see CCompilationUnitContext
 * @see CClassContext
 * @see CMethodContext
 * @see CContext
 */
public class CLoopContext
    extends CAbstractBodyContext
{

    /**
     * Construct a block context, it supports local variable allocation
     * throw statement and return statement
     * @param parent  the parent context, it must be different
     *    than null except if called by the top level
     */
    CLoopContext(CBodyContext parent, JLoopStatement self)
    {
        super(parent);

        this.self = self;

        flowState = parent.flowState | FLO_INLOOP;
    }

    public CBodyContext createBodyContext()
    {
        return new CBodyContext(this);
    }

    /**
     * remove a label from the list of labels
     * @param label  the name to be removed
     */
    public JStatement getNearestBreakableStatement()
    {
        return self;
    }

    /**
     * remove a label from the list of labels
     * @param label  the name to be removed
     */
    public JStatement getNearestContinuableStatement()
    {
        return self;
    }

    private final JLoopStatement self;
}
