/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package at.dms.kjc;

import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;

/**
 * This error display all parameters of method call.
 */
public class CMethodNotFoundError
    extends PositionedError
{
    /**
     * The expression to call the method that doesn't exist.
     */
    private final JMethodCallExpression caller;

    /**
     * Constructs an error with information about the method call.
     * XXX: refactor; name and types can be obtained from the caller object.
     *
     * @param where
     * The reference to token where error happen
     *
     * @param caller
     * The expression that made to call to the non-existant method.
     * @param name
     * The non-existant method name.
     * @param types
     * The parameter types of the method call.
     */
    public CMethodNotFoundError(TokenReference where,
                                JMethodCallExpression caller,
                                String name, CType[] types)
    {
        super(where,
              KjcMessages.METHOD_NOT_FOUND,
              buildSignature(name, types));
        /* XXX: use this:
              buildSignature(caller.getIdent(), caller.getParameterTypes());
        */

        this.caller = caller;
    }

    private static String buildSignature(String name, CType[] types)
    {
        StringBuffer buffer = new StringBuffer();

        buffer.append(name);
        buffer.append("(");
        if (types != null) {
            for (int i = 0; i < types.length; i++) {
                if (i != 0) {
                    buffer.append(", ");
                }
                buffer.append(types[i].toString());
            }
        }
        buffer.append(")");

        return buffer.toString();
    }

    /**
     * Returns the caller of the method that was not found.
     */
    public JMethodCallExpression getCaller()
    {
        return caller;
    }
}
