/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.kjc;

/**
 * This class represents all modifiers token.
 *
 * <code>public protected private static final synchronized transient
 * volatile native strictfp</code>
 */
public class CModifier
    implements Constants
{
    /**
     * Array of modifier names, in the same order as the <code>codes</code>
     * field.
     */
    private final static String[] names = {
        "public",
        "private",
        "protected",
        "static",
        "final",
        "synchronized",
        "volatile",
        "transient",
        "native",
        "", //interface not printed here
        "abstract",
        "strictfp"
    };

    /**
     * Array of modifier values, in the same order as the <code>names</code>
     * field.
     */
    private final static int[] codes = {
        ACC_PUBLIC,
        ACC_PRIVATE,
        ACC_PROTECTED,
        ACC_STATIC,
        ACC_FINAL,
        ACC_SYNCHRONIZED,
        ACC_VOLATILE,
        ACC_TRANSIENT,
        ACC_NATIVE,
        ACC_INTERFACE,
        ACC_ABSTRACT,
        ACC_STRICT
    };

    /**
     * Generate string representation of a 'modifiers' bitset.
     * @param modifiers
     * The modifiers.
     */
    public static String toString(int modifiers)
    {
        StringBuffer temp = new StringBuffer();
        for (int i = 0; i < codes.length; i++) {
            if ((modifiers & codes[i]) != 0 && (codes[i] != ACC_INTERFACE)) {
                temp.append(names[i]).append(' ');
            }
        }
        return temp.toString();
    }
}
