/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package at.dms.kjc;

/**
 * This class represents null class type in the type structure.
 */
public class CNullType
    extends CClassType
{
    /**
     * Construct a null class type.
     */
    public CNullType()
    {
        super(new CBadClass("<NULL TYPE>"));
    }

    /**
     * isAssignableTo.
     *
     * @param dest
     * The destination type.
     *
     * @return
     * Assignment is valid?
     */
    public boolean isAssignableTo(CType dest)
    {
        return dest.isReference();
    }

    /**
     * isCastableTo.
     *
     * @param dest
     * The destination type.
     *
     * @return
     * Cast is valid?
     */
    public boolean isCastableTo(CType dest)
    {
        return dest.isReference();
    }

    /**
     * implicitlyCastTo.
     *
     * @param dest
     * The expression type to cast to.
     * @return
     * Cast is valid?
     */
    public boolean implicitlyCastTo(CType dest)
    {
        return dest.isReference();
    }

    /**
     * Returns the CClass for <code>java.lang.Object</code>.
     */
    protected CClass getObject()
    {
        return CStdType.Object.getCClass();
    }

    /**
     * Returns the CClass for <code>java.lang.Object</code>.
     *
     * @return
     * The CClass for <code>java.lang.Object</code>.
     */
    public CClass getCClass()
    {
        return CStdType.Object.getCClass();
    }

    /**
     * Transforms this type to a string.
     */
    public String toString()
    {
        return "null-literal";
    }

    /**
     * Returns the size used in stack by value of this type.
     */
    public int getSize()
    {
        return 1;
    }
}
