/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package at.dms.kjc;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

public class CParseClassContext
{
    public static CParseClassContext getInstance()
    {
        return stack.size() == 0 ?
            new CParseClassContext() :
            (CParseClassContext)stack.pop();
    }

    public void release()
    {
        release(this);
    }

    public static void release(CParseClassContext context)
    {
        context.clear();
        stack.push(context);
    }

    private CParseClassContext() {}

    private void clear()
    {
        fields.clear();
        methods.clear();
        inners.clear();
        body.clear();
    }

    public void addFieldDeclaration(JFieldDeclaration decl)
    {
        fields.add(decl);
        body.add(decl);
    }

    public void addMethodDeclaration(JMethodDeclaration decl)
    {
        methods.add(decl);
    }
 
    public void addInnerDeclaration(JTypeDeclaration decl)
    {
        inners.add(decl);
    }

    public void addBlockInitializer(JClassBlock block)
    {
        body.add(block);
    }

    public JFieldDeclaration[] getFields()
    {
        JFieldDeclaration[] temp = new JFieldDeclaration[fields.size()];

        return (JFieldDeclaration[])fields.toArray(temp);
    }

    public JMethodDeclaration[] getMethods()
    {
        JMethodDeclaration[] temp = new JMethodDeclaration[methods.size()];

        return (JMethodDeclaration[])methods.toArray(temp);
    }

    public JTypeDeclaration[] getInnerClasses()
    {
        JTypeDeclaration[] temp = new JTypeDeclaration[inners.size()];

        return (JTypeDeclaration[])inners.toArray(temp);
    }

    public JPhylum[] getBody()
    {
        JPhylum[] temp = new JPhylum[body.size()];

        return (JPhylum[])body.toArray(temp);
    }

    private List fields = new ArrayList();
    private List methods = new ArrayList();
    private List inners = new ArrayList();
    private List body = new ArrayList();

    private static Stack stack = new Stack();
}
