/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package at.dms.kjc;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import at.dms.compiler.Compiler;

public class CParseCompilationUnitContext
{
    public static CParseCompilationUnitContext getInstance()
    {
        return stack.size() == 0 ?
            new CParseCompilationUnitContext() :
            (CParseCompilationUnitContext)stack.pop();
    }

    public void release()
    {
        release(this);
    }

    public static void release(CParseCompilationUnitContext context)
    {
        context.clear();
        stack.push(context);
    }

    private CParseCompilationUnitContext()
    {
        // 7.5.3 Automatic Imports
        addPackageImport(JAVA_LANG);
    }

    private void clear()
    {
        packageImports.clear();
        classImports.clear();
        typeDeclarations.clear();

        // 7.5.3 Automatic Imports
        addPackageImport(JAVA_LANG);
        pack = null;
    }

    public void setPackage(JPackageName pack)
    {
        packageName = pack == JPackageName.UNNAMED ? "" : pack.getName() + '/';
        this.pack = pack;
    }

    public void addPackageImport(JPackageImport pack)
    {
        packageImports.add(pack);
    }

    public void addClassImport(JClassImport clazz)
    {
        classImports.add(clazz);
    }

    public void addTypeDeclaration(Compiler compiler, JTypeDeclaration decl)
    {
        typeDeclarations.add(decl);
        decl.generateInterface(compiler, null, packageName);
    }

    public JPackageImport[] getPackageImports()
    {
        JPackageImport[] temp = new JPackageImport[packageImports.size()];

        return (JPackageImport[])packageImports.toArray(temp);
    }

    public JClassImport[] getClassImports()
    {
        JClassImport[] temp = new JClassImport[classImports.size()];

        return (JClassImport[])classImports.toArray(temp);
    }

    public JTypeDeclaration[] getTypeDeclarations()
    {
        JTypeDeclaration[] temp =
            new JTypeDeclaration[typeDeclarations.size()];

        return (JTypeDeclaration[])typeDeclarations.toArray(temp);
    }

    public JPackageName getPackageName()
    {
        return pack;
    }

    public static final JPackageImport JAVA_LANG =
        new JPackageImport(at.dms.compiler.TokenReference.NO_REF,
                           Constants.JAV_RUNTIME,
                           null);

    private JPackageName pack;
    private String packageName;
    private List packageImports = new ArrayList();
    private List classImports = new ArrayList();
    private List typeDeclarations = new ArrayList();

    private static Stack stack = new Stack();
}
