/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package at.dms.kjc;

import java.util.HashSet;
import java.util.Set;

import at.dms.compiler.UnpositionedError;

/**
 * This class represents a local context during checkBody
 * It follows the control flow and maintain informations about
 * variable (initialised, used, allocated), exceptions (thrown, catched)
 * It also verify that context is still reachable
 *
 * There is a set of utilities method to access fields, methods and class
 * with the name by clamping the parsing tree
 * @see CCompilationUnitContext
 * @see CClassContext
 * @see CMethodContext
 * @see CContext
 */
public class CSwitchBodyContext
    extends CAbstractBodyContext
{
    private boolean defaultExist;
    private Set labels = new HashSet();
    private JSwitchStatement statement;
    private CType switchType;

    /**
     * Construct a block context, it supports local variable allocation
     * throw statement and return statement
     * @param parent  the parent context, it must be different
     *    than null except if called by the top level
     * @param switchType the size of switch (byte, short, char...)
     */
    public CSwitchBodyContext(JSwitchStatement statement,
                              CBodyContext parent,
                              CType switchType)
    {
        super(parent);
        this.switchType = switchType;
        this.statement = statement;
        flowState = parent.flowState & FLO_INLOOP;
    }

    /**
     * close
     */
    public void close()
    {

    }

    /**
     * add a default label to this switch
     * @exception UnpositionedError this error will be positioned soon
     */
    public void addDefault() throws UnpositionedError
    {
        if (defaultExist) {
            throw new UnpositionedError(KjcMessages.SWITCH_DEFAULT_DOUBLE);
        }
        defaultExist = true;
    }

    /**
     * add a label to this switch and check it is a new one
     * @param lit  the literal value of this label
     * @exception UnpositionedError this error will be positioned soon
     */
    public void addLabel(JOrdinalLiteral lit) throws UnpositionedError
    {
        if (!lit.isAssignableTo(switchType)) {
            throw new UnpositionedError(KjcMessages.SWITCH_LABEL_OVERFLOW,
                                        lit.getValue());
        }

        if (!labels.add(lit.getValue())) {
            throw new UnpositionedError(KjcMessages.SWITCH_LABEL_EXIST,
                                        lit.getValue());
        }
    }

    /**
     * @return true is a default label has already happen
     */
    public boolean defaultExists()
    {
        return defaultExist;
    }

    /**
     * remove a label from the list of labels
     * @param label  the name to be removed
     */
    public JStatement getNearestBreakableStatement()
    {
        return statement;
    }
}
