/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package at.dms.kjc;

import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;

/**
 * A compound statement is a sequence of statements and local variable
 * declaration statements without braces.
 */
public class JCompoundStatement
    extends JStatement
{

    /**
     * Construct a node in the parsing tree
     * @param where  the line of this node in the source code
     */
    public JCompoundStatement(TokenReference where, JStatement[] body)
    {
        super(where, null);
        this.body = (body == null ? new JStatement[0] : body);
    }

    /**
     * Check statement.
     *
     * @param context the actual context of analyse
     * @exception PositionedError if the check fails
     */
    public void checkStatement(CBodyContext context)
        throws PositionedError
    {
        for (int i = 0; i < body.length; i++) {
            if (!context.isReachable()) {
                throw new CLineError(body[i].getTokenReference(),
                                     KjcMessages.STATEMENT_UNREACHABLE);
            }

            try {
                body[i].checkBody(context);
            }
            catch (CLineError e) {
                context.reportTrouble(e);
            }
        }
    }

    /**
     * Accepts the specified visitor
     * @param p  the visitor
     */
    public void accept(KjcVisitor p)
    {
        p.visitCompoundStatement(this, body);
    }

    /**
     * Generates a sequence of bytecodes
     * @param code  the code list
     */
    public void genCode(CodeSequence code)
    {
        for (int i = 0; i < body.length; i++) {
            body[i].genCode(code);
        }
    }

    private JStatement[] body;
}
