/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package at.dms.kjc;

import at.dms.classfile.NoArgInstruction;
import at.dms.compiler.PositionedError;
import at.dms.compiler.TokenReference;

/**
 * Representation of a 'null' object reference literal.
 *
 * @author DMS, Erwin Bolwidt
 */
public class JNullLiteral
    extends JLiteral
{

    /**
     * Construct a node in the parsing tree
     * @param where the line of this node in the source code
     */
    public JNullLiteral(TokenReference where)
    {
        super(where);
        /* null literals may not be considered constants according to the
         * Java Language Specification, 2nd edition.
         */
        setConstant(false);
    }

    /**
     * Compute the type of this expression (called after parsing)
     * @return the type of this expression
     */
    public CType getType()
    {
        return CStdType.Null;
    }

    /**
     * Returns null.
     */
    public Object getValue()
    {
        return null;
    }

    /**
     * Yes, the null literal is the default for its type as its the only
     * possible instance of its type.
     */
    public boolean isDefault()
    {
        return true;
    }

    /**
     * Check expression and evaluate and alter context.
     *
     * @param context the context of the check.
     *
     * @return a pure java expression including promote node.
     */
    public JExpression checkExpression(CExpressionContext context)
    {
        return this;
    }

    /**
     * Accepts the specified visitor.
     *
     * @param p the visitor
     */
    public void accept(KjcVisitor p)
    {
        p.visitNullLiteral();
    }

    /**
     * Generates a sequence of bytecodes.
     *
     * @param code the code list
     */
    public void genCode(CodeSequence code)
    {
        code.setLineNumber(getTokenReference().getLine());

        code.plantInstruction(new NoArgInstruction(opc_aconst_null));
    }
}
