/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package at.dms.kjc;

import java.util.ArrayList;
import java.util.List;

import at.dms.compiler.Compiler;
import at.dms.compiler.CWarning;
import at.dms.compiler.TokenReference;
import at.dms.compiler.JavaStyleComment;

/**
 * This class represents the "package at.dms.kjc" statement
 */
public class JPackageImport extends JPhylum
{

    /**
     * construct a package name
     *
     * @param where  the token reference of this node
     * @param name  the package name
     */
    public JPackageImport(TokenReference where,
                          String name,
                          JavaStyleComment[] comments)
    {
        super(where);

        this.name = name;
        this.comments = comments;
    }

    /**
     * Returns the package name defined by this declaration.
     *
     * @return the package name defined by this declaration
     */
    public String getName()
    {
        return name;
    }

    /**
     * States that specified class in imported package is used.
     * @param clazz  the class that is used.
     */
    public void setClassUsed(String clazz)
    {
        if (classesUsed == null) {
            classesUsed = new ArrayList();
        }
        classesUsed.add(clazz);
    }

    /**
     * Check expression and evaluate and alter context
     */
    public void checkBody(Compiler compiler, JPackageName pack)
    {
        if (getTokenReference() != TokenReference.NO_REF) {
            if (classesUsed == null) {
                compiler.reportTrouble
                    (new CWarning(getTokenReference(),
                                  KjcMessages.UNUSED_PACKAGE_IMPORT,
                                  name.replace('/', '.'),
                                  null));
            }
            else if (classesUsed.size() < 5) {
                StringBuffer buffer = new StringBuffer();

                for (int i = 0; i < classesUsed.size(); i++) {
                    if (i != 0) {
                        buffer.append(", ");
                    }
                    buffer.append(name).append("/");
                    String className =
                        ((String)classesUsed.get(i)).replace('/', '.');
                    buffer.append(className);
                }
                compiler.reportTrouble
                    (new CWarning(getTokenReference(),
                                  KjcMessages.REPLACE_PACKAGE_IMPORT,
                                  name.replace('/', '.'),
                                  buffer.toString()));
            }
    
            if (name.equals("java/lang")) {
                compiler.reportTrouble
                    (new CWarning(getTokenReference(),
                                  KjcMessages.IMPORT_JAVA_LANG,
                                  name));
            }
      
            if (name.equals(pack.getName())) {
                compiler.reportTrouble
                    (new CWarning(getTokenReference(),
                                  KjcMessages.IMPORT_CURRENT_PACKAGE,
                                  name));
            }
        }
    }

    /**
     * Accepts the specified visitor
     * @param p  the visitor
     */
    public void accept(KjcVisitor p)
    {
        if (comments != null) {
            p.visitComments(comments);
        }
        p.visitPackageImport(name);
    }

    private final String name;
    private final JavaStyleComment[] comments;
    private List classesUsed;
}
