/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 * Copyright (C) 2001    Erwin Bolwidt, Haarlem, The Netherlands.
 *
 * The original author of this file is DMS; refactorings, bug fixes and
 * extensions were done by Erwin Bolwidt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package at.dms.kjc;

import at.dms.compiler.TokenReference;
import at.dms.compiler.JavaStyleComment;

/**
 * This class represents the "package at.dms.kjc" statement
 */
public class JPackageName
    extends JPhylum
{

    /**
     * The unnamed package (JLS 7.4.2)
     */
    public static JPackageName UNNAMED =
        new JPackageName(TokenReference.NO_REF, "", null);


    /**
     * construct a package name
     *
     * @param where  the token reference of this node
     * @param name  the package name
     */
    public JPackageName(TokenReference where,
                        String name,
                        JavaStyleComment[] comments)
    {
        super(where);

        this.name = name.intern();
        this.comments = comments;
    }

    /**
     * Returns the package name defined by this declaration.
     *
     * @return the package name defined by this declaration
     */
    public String getName()
    {
        return name;
    }

    /**
     * Accepts the specified visitor
     * @param p  the visitor
     */
    public void accept(KjcVisitor p)
    {
        if (comments != null) {
            p.visitComments(comments);
        }
        if (!name.equals("")) {
            p.visitPackageName(name.replace('/', '.'));
        }
    }

    private final String name;
    private final JavaStyleComment[] comments;
}

