/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package at.dms.ikjc;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.io.PrintWriter;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.net.Socket;

/**
 * This class implements the client side of the kjc incremental compiler.
 */
public class IkjcClient
{
    /**
     * Host to connect to / connected to.
     */
    private static InetAddress host;

    /**
     * Port to connect to / connected to.
     */
    private static int port;

    /**
     * Socket connection to compiler server.
     */
    private static Socket kkSocket;

    /**
     * Write stream to the compiler server.
     */
    private static PrintWriter out;

    /**
     * Read stream from the compiler server.
     */
    private static BufferedReader in;

    static String TERM_STRING = "!@#$%^&*()";

    /**
     * Arguments are just passed to kjc except if there is a -port that set
     * the port for this session.
     */
    public static void main(String[] args)
        throws IOException
    {
        host = InetAddress.getLocalHost();
        port = 4444;

        args = parseIkjcArgs(args);

        // CONNECT
        try {
            kkSocket = new Socket(host, port);
            out = new PrintWriter(kkSocket.getOutputStream(), true);
            in = new BufferedReader(new InputStreamReader
                (kkSocket.getInputStream()));
        }
        catch (UnknownHostException e) {
            System.err.println("Host unknown: " + host);
            System.exit(1);
        }
        catch (IOException e) {
            System.err.println("Couldn't get I/O for the connection to: "
                               + host + ":" + port);
            System.err.println("Make sure that the server is running, else "
                               + "launch it with:");
            System.err.println(" kjcServer [-port <port-number>]");
            System.exit(1);
        }

        // pass working directory
        out.println(System.getProperty("user.dir"));

        // pass arguments
        for (int i = 0; i < args.length; i++) {
            out.println(args[i]);
        }
        out.println();

        // read results
        String  fromServer;
        boolean compileOkay = false;
        while ((fromServer = in.readLine()) != null) {
            if (fromServer.endsWith(IkjcClient.TERM_STRING)) {
                compileOkay = new Boolean(in.readLine()).booleanValue();
                break;
            }
            else {
                System.out.println(fromServer);
            }
        }

        out.close();
        in.close();
        kkSocket.close();
        System.exit(compileOkay ? 0 : 1);
    }

    /**
     * Parses the argument list and sets flags.
     * @return the arguments to pass to the remote compiler
     */
    private static String[] parseIkjcArgs(String[] args)
    {
        int i;

        for (i = 0; i < args.length; i++) {
            if (args[i].equals("-h")) {
                i += 1;
                if (i == args.length) {
                    System.err.println("argument for option -h is missing");
                    System.exit(1);
                }

                try {
                    host = InetAddress.getByName(args[i]);
                }
                catch (Exception e) {
                    System.err.println("Bad host: " + args[i]);
                    System.exit(1);
                }
            }
            else if (args[i].equals("-p")) {
                i += 1;
                if (i == args.length) {
                    System.err.println("argument for option -p is missing");
                    System.exit(1);
                }

                try {
                    port = Integer.parseInt(args[i]);
                }
                catch (Exception e) {
                    System.err.println("Bad port number: " + args[i]);
                    System.exit(1);
                }
            }
            else if (args[i].equals("-help")) {
                help();
            }
            else {
                break;
            }
        }

        if (i == args.length) {
            System.err.println("No arguments given.");
            System.exit(1);
        }

        String[] result = new String[args.length - i];

        System.arraycopy(args, i, result, 0, args.length - i);
        return result;
    }

    /**
     * Shows a help message.
     */
    private static void help()
    {
        PrintStream err = System.err;
        err.println("Usage: ikjc [-h host] [-p port] [-help] compiler "
                    + "[compiler-args]");
        err.println("    -h host:  set the host name for remote compiling");
        err.println("    -p port:  set the port number of server (int)");
        err.println("    compiler: the compiler to use (one of kjc, xkjc, "
                    + "kopic)");
    }
}
