/*
 * Copyright (C) 1990-99 DMS Decision Management Systems Ges.m.b.H.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * $Id: ListCache.java,v 1.1.1.1 2001/02/15 17:27:51 ejb Exp $
 */

package at.dms.util;

import java.util.ArrayList;
import java.util.List;
import java.util.LinkedList;

/**
 * This class implements a cache of lists
 */
public class ListCache
{
    /**
     * Special value for requesting a "small" list.
     */
    public static final boolean SMALL = false;

    /**
     * Special value for requesting a "big" list.
     */
    public static final boolean BIG = true;

    private static final int SMALL_SIZE = 5;
    private static final int BIG_SIZE = 100;

    /**
     * Returns a list.
     * @param size
     *     Either BIG or SMALL, indicating how large the requester thinks that
     *     the capacity of the list should be.
     */
    public static synchronized List request(boolean size)
    {
        if (size == SMALL) {
            if (smalls.isEmpty()) {
                return new ArrayList(SMALL_SIZE);
            }
            else {
                return (List)smalls.removeFirst();
            }
        }
        else {
            if (bigs.isEmpty()) {
                return new ArrayList(BIG_SIZE);
            }
            else {
                return (List)bigs.removeFirst();
            }
        }
    }

    /**
     * Releases a char array.
     */
    public static void release(List list)
    {
        // there is no ArrayList.capacity, so just use size()
        if (list.size() >= BIG_SIZE) {
            list.clear();
            bigs.addLast(list);
        }
        else {
            list.clear();
            smalls.addLast(list);
        }
    }

    // ----------------------------------------------------------------------
    // PRIVATE DATA MEMBERS
    // ----------------------------------------------------------------------

    private static LinkedList smalls = new LinkedList();
    private static LinkedList bigs = new LinkedList();
}
